APP_TEXT = """
import click
from {app_name} import app

@click.group()
def cli():
    \"\"\"{app_name}'s cli\"\"\"

@cli.command(name="start")
@click.option("--debug", is_flag=True, help="Run app in debug mode.")
@click.option("--port", default=5000, help="Port to start the webserver.")
def _start(debug, port):
    print("[+] Routes")
    for rule in app.url_map.iter_rules():
        if 'static/<path:filename>' not in str(rule):
            print(f'    http://localhost:{port}{rule}')
    print('\\n')

    app.run(debug=debug, port=port)

if __name__ == "__main__":
    cli()
"""

INIT_TEXT = """
import os
import importlib
from quart import Quart, render_template, make_response
from .config import config

app = Quart(__name__)
app.config.from_object(config['development']) # Change it to production for production use

_apps = {}
apps_folder = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'apps')

if os.path.exists(apps_folder):
    for path in os.listdir(apps_folder):
        module_path = os.path.join(apps_folder, path)
        if os.path.isdir(module_path) and not path.startswith('__'):
            module = importlib.import_module(f'.{path}', package=f"{app.name}.apps")
            _apps[path] = getattr(module, path)

for _name, _module in _apps.items():
    app.register_blueprint(_module, url_prefix=f"/{_name}")

@app.route('/')
async def index():
    template = await render_template("index.html")
    response = await make_response(template)

    return response
"""

CONFIG_TEXT = """
class Config:
    SECRET_KEY = "{secret}"

    @staticmethod
    def init_app(app):
        pass

class DevelopmentConfig(Config):
    DEBUG = True

class ProductionConfig(Config):
    DEBUG = False


config = {
    "development": DevelopmentConfig,
    "production": ProductionConfig,
}
"""

INDEX_TEXT = """
<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <link rel="shortcut icon" href="https://pgjones.gitlab.io/quart/_static/logo_short.png" type="image/png">
    <link rel="stylesheet" href="{{ url_for('static', filename='style.css') }}">
    <title>Quart</title>
</head>
<body>
    <div class="header container">
        <div class="title">
            Quart
        </div>

        <a href="https://pgjones.gitlab.io/quart/" target="_blank">Documentation</a>
    </div>


    <div class="content container">
        <img src="https://pgjones.gitlab.io/quart/_images/logo.png" alt="Quart">
        <p><span class="qspan">Quart</span> is a Python web microframework</p>
    </div>
</body>
</html>
"""

STYLE_TEXT = """
.container {
    max-width: 960px;
    margin: 0 auto;
    padding-left: 20px;
    padding-right: 20px;
}

.header {
    display: flex;
    justify-content: space-between;
    align-items: center;
    height: 60px;
}

.header .title {
    font-size: 30px;
    color: #ec5a3b;
}

.header a {
    color: #008dff;
    text-decoration: none;
    font-size: 20px;
}

.content {
    text-align: center;
    width: 100%;
    height: 100vh;
    font-size: 20px;
}

.content img {
    width: 300px;
    max-width: 100%;
    margin-top: 50px;
}

.qspan {
    color: #ec5a3b;
}
"""

BLP_TEXT = """
from quart import Blueprint

{app_name} = Blueprint("{app_name}", __name__)

from .views import *
from .errors import *
"""

VIEWS_TEXT = """
from . import {app_name}
from quart import render_template, make_response

@{app_name}.route('/')
async def index():
    template = await render_template('{app_name}/index.html')
    response = await make_response(template)

    return response
"""

ERRORS_TEXT = """
from . import {app_name}

@{app_name}.app_errorhandler(404)
async def not_found(e):
    return "This route doesn't exist", 404
"""

BLP_STYLE = """
.qspan {
    color: #ec5a3b;
}
"""

BLP_INDEX = """
<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <link rel="stylesheet" href="{{ url_for('static', filename='{app_name}/style.css') }}">
    <title>{app_name}</title>
</head>
<body>
    <p><span class="qspan">{app_name}</span> app's index view</p>
</body>
</html>
"""



import os
import shutil


def empty_folder(path):
    """Clears all file of a folder"""
    for filename in os.listdir(path):
        file_path = os.path.join(path, filename)
        try:
            if os.path.isfile(file_path) or os.path.islink(file_path):
                os.unlink(file_path)
            elif os.path.isdir(file_path):
                shutil.rmtree(file_path)
        except Exception as e:
            print("Failed to delete %s. Reason: %s" % (file_path, e))


def go_back():
    """Go back from current directory"""
    os.chdir("..")


def create_file(name, content, replaces={}):
    """Creates a file with given content"""
    for key, value in replaces.items():
        content = content.replace(key, value)

    with open(name, "w") as file:
        file.write(content.strip())



import os
import click
from .contents import *
from slugify import slugify
from .helpers import create_file
from .context import make_dir, open_dir

TEMPLATES_DIR = "templates"
STATIC_DIR = "static"


@click.group(chain=True)
def cli():
    """Quart-admin"""


@cli.command("startproject")
@click.option("--name", "-n", help="Project name", default="")
def start_project(name):
    """Creates a quart project"""
    if not name:
        print("Project Name : ", end="")
        name = input()

    folder_name = slugify(name)
    app_name = folder_name.replace("-", "_")

    with make_dir(folder_name, back=False, empty=True) as _:
        create_file("app.py", APP_TEXT, {"{app_name}": app_name})

        with make_dir(app_name, back=False) as _:
            create_file("__init__.py", INIT_TEXT, {"{app_name}": app_name})
            create_file("config.py", CONFIG_TEXT, {"{secret}": app_name})

            with make_dir(TEMPLATES_DIR, back=True) as _:
                create_file("index.html", INDEX_TEXT)

            with make_dir(STATIC_DIR, back=True) as _:
                create_file("style.css", STYLE_TEXT)

    print(
        f"Quart project created. Use `cd {folder_name}` and open with your favourite code editor."
    )


@cli.command("startapp")
@click.option("--name", "-n", help="App/Blurprint name", default="")
def start_app(name):
    """Creates a app inside a quart app"""
    if not name:
        print("App Name : ", end="")
        name = input()

    app_name = slugify(name).replace("-", "_")
    folders = [f.path for f in os.scandir(os.getcwd()) if f.is_dir()]

    project_dir = ""
    is_project = False

    for folder in folders:
        files = os.listdir(folder)
        if "config.py" in files and "__init__.py" in files:
            is_project = True
            project_dir = folder
            break

    if not is_project:
        return print(
            "No quart project found on this directory. Can't create app here. Navigate to a project directory or create a project."
        )

    os.chdir(project_dir)

    with make_dir("apps", back=True, empty=False) as _:
        with make_dir(app_name, back=True) as _:
            create_file("__init__.py", BLP_TEXT, {"{app_name}": app_name})
            create_file("views.py", VIEWS_TEXT, {"{app_name}": app_name})
            create_file("errors.py", ERRORS_TEXT, {"{app_name}": app_name})

    with open_dir(STATIC_DIR, back=True) as _:
        with make_dir(app_name, back=True) as _:
            create_file("style.css", BLP_STYLE)

    with open_dir(TEMPLATES_DIR, back=True) as _:
        with make_dir(app_name, back=True) as _:
            create_file("index.html", BLP_INDEX, {"{app_name}": app_name})

    print(f"App {name} created successfully. Will be accessible on '/{name}' route")



import os
from .helpers import empty_folder, go_back
from contextlib import contextmanager


@contextmanager
def open_dir(name, back=True):
    """Opens a directory"""
    if not os.path.exists(name):
        os.mkdir(name)

    os.chdir(name)

    yield None

    if back:
        go_back()


@contextmanager
def make_dir(name, back=True, empty=True):
    """Makes a directory"""
    if not os.path.exists(name):
        os.mkdir(name)

    os.chdir(name)

    if empty:
        empty_folder(os.getcwd())

    yield None

    if back:
        go_back()



import pathlib
from setuptools import setup, find_packages

here = pathlib.Path(__file__).parent.resolve()

long_description = (here / "README.md").read_text(encoding="utf-8")

setup(
    name="quart-admin",
    version="1.0.3",
    description="A tool to create quart projects faster with a scalable setup",  # Optional
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/shahriyardx/quart-admin",
    author="Md Shahriyar Alam",
    author_email="contact@shahriyar.dev",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Build Tools",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3 :: Only",
    ],
    entry_points={"console_scripts": ["quart-admin = quart_admin:cli"]},
    keywords="quart, quart-admin",
    packages=find_packages(),
    python_requires=">=3.6, <4",
    install_requires=["quart==0.15.1", "click==7.1.2", "python-slugify"],
    project_urls={
        "Bug Reports": "https://github.com/shahriyardx/quart-admin/issues",
        "Source": "https://github.com/shahriyardx/quart-admin/",
    },
)
